@@ -36,8 +36,8 @@ from merlin.core.dispatch import (
     make_df,
     pull_apart_list,
 )
+from merlin.io.shuffle import shuffle_df
 from merlin.models.loader.dataframe_iter import DataFrameIter
-from merlin.models.loader.shuffle import _shuffle_df
 from merlin.schema import Tags
 
 
@@ -132,7 +132,7 @@ class ChunkQueue:
             chunks.reset_index(drop=True, inplace=True)
             chunks, spill = self.get_batch_div_chunk(chunks, self.dataloader.batch_size)
             if self.shuffle:
-                chunks = _shuffle_df(chunks)
+                chunks = shuffle_df(chunks)
 
             if len(chunks) > 0:
                 chunks = self.dataloader.make_tensors(chunks, self.dataloader._use_nnz)
@@ -223,10 +223,14 @@ class DataLoader:
         self._epochs = 1
 
         self.cat_names = cat_names or (
-            self.schema.select_by_tag(Tags.CATEGORICAL).column_names if self.schema else []
+            self.schema.select_by_tag(Tags.CATEGORICAL).excluding_by_tag(Tags.TARGET).column_names
+            if self.schema
+            else []
         )
         self.cont_names = cont_names or (
-            self.schema.select_by_tag(Tags.CONTINUOUS).column_names if self.schema else []
+            self.schema.select_by_tag(Tags.CONTINUOUS).excluding_by_tag(Tags.TARGET).column_names
+            if self.schema
+            else []
         )
         self.label_names = label_names or (
             self.schema.select_by_tag(Tags.TARGET).column_names if self.schema else []
@@ -569,6 +573,8 @@ class DataLoader:
             if len(column_names) == 0:
                 tensors.append(None)
                 continue
+            if hasattr(column_names, "column_names"):
+                column_names = column_names.column_names
 
             gdf_i = gdf[column_names]
             gdf.drop(columns=column_names, inplace=True)
