@@ -14,6 +14,7 @@
 # limitations under the License.
 #
 import contextlib
+import logging
 import os
 
 import dask.dataframe as dd
@@ -22,10 +23,14 @@ import tensorflow as tf
 from packaging import version
 
 from merlin.core.dispatch import HAS_GPU
+from merlin.io import Dataset
 from merlin.models.loader.backend import DataLoader
 from merlin.models.loader.tf_utils import get_dataset_schema_from_feature_columns
+from merlin.models.utils.schema_utils import select_targets
 from merlin.schema import Tags
 
+LOG = logging.getLogger("merlin.models")
+
 if version.parse(tf.__version__) < version.parse("2.3.0"):
     try:
         from tfdlpack import from_dlpack
@@ -36,7 +41,6 @@ if version.parse(tf.__version__) < version.parse("2.3.0"):
 else:
     from tensorflow.experimental.dlpack import from_dlpack
 
-
 # pylint has issues with TF array ops, so disable checks until fixed:
 # https://github.com/PyCQA/pylint/issues/3613
 # pylint: disable=no-value-for-parameter,unexpected-keyword-arg,redundant-keyword-arg
@@ -49,12 +53,12 @@ dd_engine = {
 }
 
 
-def _validate_dataset(paths_or_dataset, batch_size, buffer_size, engine, reader_kwargs):
+def _validate_dataset(paths_or_dataset, batch_size, buffer_size, engine, device, reader_kwargs):
     # TODO: put this in parent class and allow
     # torch dataset to leverage as well?
 
     # if a dataset was passed, just return it
-    if hasattr(paths_or_dataset, "schema") or isinstance(paths_or_dataset, dd.DataFrame):
+    if hasattr(paths_or_dataset, "schema"):
         return paths_or_dataset
 
     # otherwise initialize a dataset
@@ -76,15 +80,27 @@ def _validate_dataset(paths_or_dataset, batch_size, buffer_size, engine, reader_
     if not engine:
         # default engine is parquet
         engine = "parquet"
-    return dd_engine[engine](files)
+
+    cpu = device and "cpu" in device
+
+    return Dataset(files, engine=engine, cpu=cpu)
 
 
-def _validate_schema(feature_columns, cat_names, cont_names, schema=None):
+def _validate_schema(feature_columns, cat_names, cont_names, label_names, schema=None):
     _uses_feature_columns = feature_columns is not None
     _uses_explicit_schema = (cat_names is not None) or (cont_names is not None)
 
-    cat_tag_names = schema.select_by_tag([Tags.CATEGORICAL]).column_names if schema else []
-    cont_tag_names = schema.select_by_tag([Tags.CONTINUOUS]).column_names if schema else []
+    cat_tag_names = (
+        schema.select_by_tag([Tags.CATEGORICAL]).excluding_by_tag(Tags.TARGET).column_names
+        if schema
+        else []
+    )
+    cont_tag_names = (
+        schema.select_by_tag([Tags.CONTINUOUS]).excluding_by_tag(Tags.TARGET).column_names
+        if schema
+        else []
+    )
+    label_names = label_names or []
     _uses_dataset_schema = cat_tag_names or cont_tag_names
 
     if _uses_feature_columns and _uses_explicit_schema:
@@ -92,15 +108,18 @@ def _validate_schema(feature_columns, cat_names, cont_names, schema=None):
             "Passed `feature_column`s and explicit column names, must be one or the other"
         )
     elif _uses_feature_columns:
-        return get_dataset_schema_from_feature_columns(feature_columns)
+        cat_names, cont_names = get_dataset_schema_from_feature_columns(feature_columns)
+
+        return cat_names, cont_names, label_names
     elif _uses_explicit_schema:
         cat_names = cat_names or []
         cont_names = cont_names or []
-        return cat_names, cont_names
+
+        return cat_names, cont_names, label_names
     elif _uses_dataset_schema:
-        cat_tag_names = cat_tag_names or []
-        cont_tag_names = cont_tag_names or []
-        return cat_tag_names, cont_tag_names
+        label_names = label_names or select_targets(schema).column_names
+
+        return cat_tag_names, cont_tag_names, label_names
     else:
         raise ValueError(
             "Must either pass a list of TensorFlow `feature_column`s "
@@ -114,8 +133,14 @@ def _get_schema(dataset):
     return None
 
 
-class Dataset(tf.keras.utils.Sequence, DataLoader):
+class BatchedDataset(tf.keras.utils.Sequence, DataLoader):
     """
+    Override class to customize data loading for backward compatibility with
+    older NVTabular releases.
+
+    In most cases, when you use the `merlin.io.Dataset` class, data loading
+    for model training and evaluation is performed automatically by Merlin Models.
+
     Infinite generator used to asynchronously iterate through CSV or Parquet
     dataframes on GPU by leveraging an NVTabular `Dataset`. Applies preprocessing
     via NVTabular `Workflow` objects and outputs tabular dictionaries of TensorFlow
@@ -167,34 +192,34 @@ class Dataset(tf.keras.utils.Sequence, DataLoader):
 
     Parameters
     -------------
-    - paths_or_dataset: str or list(str)
+    paths_or_dataset: str or list(str)
         Either a string representing a file pattern (see `tf.glob` for
         pattern rules), a list of filenames to be iterated through, or
         a Dataset object, in which case `buffer_size`, `engine`, and
         `reader_kwargs` will be ignored
-    - batch_size: int
+    batch_size: int
         Number of samples to yield at each iteration
-    - label_names: list(str)
+    label_names: list(str)
         Column name of the target variable in the dataframe specified by
         `paths_or_dataset`
-    - feature_columns: list(tf.feature_column) or None
+    feature_columns: list(tf.feature_column) or None
         A list of TensorFlow feature columns representing the inputs
         exposed to the model to be trained. Columns with parent columns
         will climb the parent tree, and the names of the columns in the
         unique set of terminal columns will be used as the column names.
         If left as None, must specify `cat_names` and `cont_names`
-    - cat_names: list(str) or None
+    cat_names: list(str) or None
         List of categorical column names. Ignored if `feature_columns` is
         specified
-    - cont_names: list(str) or None
+    cont_names: list(str) or None
         List of continuous column names. Ignored if `feature_columns` is
         specified
-    - engine: {'csv', 'parquet', None}, default None
+    engine: {'csv', 'parquet', None}, default None
         String specifying the type of read engine to use. If left as `None`,
         will try to infer the engine type from the file extension.
-    - shuffle: bool, default True
+    shuffle: bool, default True
         Whether to shuffle chunks of batches before iterating through them.
-    - buffer_size: float or int
+    buffer_size: float or int
         If `0 <  buffer_size < 1`, `buffer_size` will refer to the fraction of
         total GPU memory to occupy with a buffered chunk. If `1 < buffer_size <
         batch_size`, the number of rows read for a buffered chunk will
@@ -203,20 +228,20 @@ class Dataset(tf.keras.utils.Sequence, DataLoader):
         the last chunk in a dataset, which will, in general, be smaller).
         Larger chunk sizes will lead to more efficiency and randomness,
         but require more memory.
-    - device: None
+    device: None
         Which GPU device to load from. Ignored for now
-    - parts_per_chunk: int
+    parts_per_chunk: int
         Number of dataset partitions with size dictated by `buffer_size`
         to load and concatenate asynchronously. More partitions leads to
         better epoch-level randomness but can negatively impact throughput
-    - reader_kwargs: dict
+    reader_kwargs: dict
         extra kwargs to pass when instantiating the underlying
         `nvtabular.Dataset`
     sparse_list : list(str) or None
         list with column names of columns that should be represented as sparse tensors
     sparse_max : dict
         dictionary of key: column_name + value: integer representing max sequence length for column
-    sparse_dense : bool
+    sparse_as_dense : bool
         bool value to activate transforming sparse tensors to dense
     """
 
@@ -242,15 +267,17 @@ class Dataset(tf.keras.utils.Sequence, DataLoader):
         drop_last=False,
         sparse_names=None,
         sparse_max=None,
+        multi_label_as_dict=True,
         sparse_as_dense=False,
         schema=None,
     ):
         dataset = _validate_dataset(
-            paths_or_dataset, batch_size, buffer_size, engine, reader_kwargs
+            paths_or_dataset, batch_size, buffer_size, engine, device, reader_kwargs
         )
-        schema = _get_schema(dataset) if not schema else schema
-        cat_names, cont_names = _validate_schema(
-            feature_columns, cat_names, cont_names, schema=schema
+        if schema:
+            dataset.schema = schema
+        cat_names, cont_names, label_names = _validate_schema(
+            feature_columns, cat_names, cont_names, label_names, schema=dataset.schema
         )
 
         device = device or 0
@@ -274,6 +301,8 @@ class Dataset(tf.keras.utils.Sequence, DataLoader):
             sparse_as_dense=sparse_as_dense,
         )
         self._map_fns = []
+        if len(label_names) > 1 and multi_label_as_dict:
+            self._map_fns.append(lambda X, y: (X, dict(zip(label_names, y))))
 
     def __len__(self):
         """
@@ -468,3 +497,39 @@ class DatasetValidator(tf.keras.callbacks.Callback):
         logs.update(set_logs)
         print(set_logs)
         return logs
+
+
+def sample_batch(
+    data: Dataset,
+    batch_size: int,
+    shuffle: bool = False,
+    include_targets: bool = True,
+    to_dense: bool = False,
+):
+    """Util function to generate a batch of input tensors from a merlin.io.Dataset instance
+
+    Parameters
+    ----------
+    data: merlin.io.dataset
+        A Dataset object.
+    batch_size: int
+        Number of samples to return.
+    shuffle: bool
+        Whether to sample a random batch or not, by default False.
+    include_targets: bool
+        Whether to include the targets in the returned batch, by default True.
+    to_dense: bool
+        Whether to convert the tuple of sparse tensors into dense tensors, by default False.
+    Returns:
+    -------
+    batch: Dict[tf.tensor]
+        dictionary of input tensors.
+    """
+    from merlin.models.tf.blocks.core.transformations import AsDenseFeatures
+
+    inputs, targets = next(iter(BatchedDataset(data, batch_size=batch_size, shuffle=shuffle)))
+    if to_dense:
+        inputs = AsDenseFeatures()(inputs)
+    if not include_targets:
+        return inputs
+    return inputs, targets
