@@ -14,10 +14,9 @@
 # limitations under the License.
 #
 
-from .listwise_losses import CategoricalCrossEntropy, SparseCategoricalCrossEntropy
-from .loss_base import LossType, loss_registry
-from .pairwise_losses import (
-    AdaptiveHingeLoss,
+from merlin.models.tf.losses.base import LossType, loss_registry
+from merlin.models.tf.losses.listwise import CategoricalCrossEntropy, SparseCategoricalCrossEntropy
+from merlin.models.tf.losses.pairwise import (
     BPRLoss,
     BPRmaxLoss,
     HingeLoss,
@@ -30,7 +29,6 @@ from .pairwise_losses import (
 __all__ = [
     "CategoricalCrossEntropy",
     "SparseCategoricalCrossEntropy",
-    "AdaptiveHingeLoss",
     "BPRLoss",
     "BPRmaxLoss",
     "HingeLoss",
